import { Construct, IResource, Resource } from '@aws-cdk/cdk';
import { Parameters } from './props';
/**
 * A cluster parameter group
 */
export interface IClusterParameterGroup extends IResource {
    /**
     * Name of this parameter group
     */
    readonly parameterGroupName: string;
}
/**
 * Properties to reference a cluster parameter group
 */
export interface ClusterParameterGroupImportProps {
    readonly parameterGroupName: string;
}
/**
 * Properties for a cluster parameter group
 */
export interface ClusterParameterGroupProps {
    /**
     * Database family of this parameter group
     */
    readonly family: string;
    /**
     * Description for this parameter group
     */
    readonly description: string;
    /**
     * The parameters in this parameter group
     *
     * @default - No parameters.
     */
    readonly parameters?: Parameters;
}
/**
 * Defina a cluster parameter group
 *
 * @resource AWS::RDS::DBClusterParameterGroup
 */
export declare class ClusterParameterGroup extends Resource implements IClusterParameterGroup {
    /**
     * Import a parameter group
     */
    static fromParameterGroupName(scope: Construct, id: string, parameterGroupName: string): IClusterParameterGroup;
    readonly parameterGroupName: string;
    private readonly parameters;
    constructor(scope: Construct, id: string, props: ClusterParameterGroupProps);
    /**
     * Set a single parameter in this parameter group
     */
    setParameter(key: string, value: string | undefined): void;
    /**
     * Remove a previously-set parameter from this parameter group
     */
    removeParameter(key: string): void;
    /**
     * Validate this construct
     */
    protected validate(): string[];
}
